package com.citygis.utils;

import com.citygis.pojo.Para;
import com.citygis.service.KafkaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.YearMonth;

@Service
public class DataUtil {

    @Resource
    KafkaService kafkaService;


    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    //15张慢病表
    static String[] tableNemes = {"cdcmb_cdc_fr_jyxx",
            "cdcmb_cdc_fr_sccy",
            "cdcmb_cdc_fr_mbdj",
            "cdcmb_cdc_fr_mbdj_jzs",
            "cdcmb_cdc_fr_mbsf_fyxx",
            "cdcmb_cdc_fr_mbsf_nczgw_jbs",
            "cdcmb_cdc_fr_mbsfk",
            "cdcmb_cdc_fr_tnbscdj",
            "cdcmb_cdc_fr_tzcl",
            "cdcmb_cdc_fr_zg",
            "cdcmb_cdc_tb_sfjh",
            "cdcmb_cdc_tb_xt_deadcard",
            "cdcmb_cdc_tb_xt_gldxjbxx",
            "cdcmb_cdc_xt_sczzjg",
            "cdcmb_cdc_fr_gxy_alert"};

    public void saveRedis(String key) {
        //判断redis是否存在指定key，有就累加，没有就新建
        if (Boolean.TRUE.equals(redisTemplate.hasKey(key))) {
            Integer value = (int) redisTemplate.opsForValue().get(key);
            Integer i = value + 1;
            redisTemplate.opsForValue().set(key, i);
        } else {
            redisTemplate.opsForValue().set(key, 1);
        }
    }

    /**
     * 每天凌晨一点往数据库存储昨天的数据
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void saveDataBaseByKafkaData() {
        LocalDate currentDate = LocalDate.now(); // 获取当前日期
        LocalDate yesterday = currentDate.minusDays(1); // 获取昨天的日期

        int year = yesterday.getYear(); // 获取年份
        int month = yesterday.getMonthValue(); // 获取月份
        int day = yesterday.getDayOfMonth(); // 获取日

        LocalDate date = LocalDate.of(year, month, day);
        for (String tableNeme : tableNemes) {
            String redisKeyXF = date + "_" + tableNeme + "_XF";
            String redisKeyFS = date + "_" + tableNeme + "_FS";
//            System.out.println(redisKeyXF);
//            System.out.println(redisKeyFS);


            Object xf = redisTemplate.opsForValue().get(redisKeyXF);
            Object fs = redisTemplate.opsForValue().get(redisKeyFS);

            int valueXF = 0;
            int valueFS = 0;
            if (xf != null) {
                // 进行拆箱操作
                valueXF = (int) xf;
                // 使用intValue进行后续操作
            }
            if (fs != null) {
                valueFS = (int) fs;
            }

            Para para = new Para();
            para.setSendCount(String.valueOf(valueFS));
            para.setReceiveCount(String.valueOf(valueXF));
            para.setTime(date.toString());
            para.setTabName(tableNeme);
            kafkaService.addKafkaCount(para);
            System.out.println(date+"---定时存储执行");
        }
    }

    /**
     * 每月28号删除上个月的数据
     */
    @Scheduled(cron = "0 0 0 28 * ?")
    public void deleteLastMonthRedisData() {

        YearMonth currentYearMonth = YearMonth.now(); // 获取当前年月
        YearMonth lastMonth = currentYearMonth.minusMonths(1); // 获取上个月的年月

        //获取昨天的日期
        int year = lastMonth.getYear(); // 获取年份
        int month = lastMonth.getMonthValue(); // 获取月份
        int days = lastMonth.lengthOfMonth(); // 获取当月的天数

        for (int day = 1; day <= days; day++) {
            LocalDate date = LocalDate.of(year, month, day);
            //删除上个月redis数据
            for (String tableNeme : tableNemes) {
                String redisKeyXF = date + "_" + tableNeme + "_XF";
                String redisKeyFS = date + "_" + tableNeme + "_FS";
                //删除key
                redisTemplate.delete(redisKeyXF);
                redisTemplate.delete(redisKeyFS);
            }
        }
        System.out.println("定时删除执行");
    }
}
