package com.citygis.utils;
import com.sun.management.OperatingSystemMXBean;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.lang.management.ThreadMXBean;
import java.net.*;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Enumeration;


/**
 * @Package com.citygis.utils
 * @Author zyl
 * @Date 2024/1/22 0022 9:54
 * @description: 监控网络带宽
 */

public class SystemMonitor {

    public static void main(String[] args) throws Exception {
        String host = "example.com"; // 替换为实际的主机名或IP地址
        int port = 80; // 替换为实际的端口号

        try (Socket socket = new Socket()) {
            socket.connect(new InetSocketAddress(host, port), 1000); // 连接指定主机和端口，设置超时时间为1秒

            // 发送部分
            long startSendTime = System.nanoTime();
            socket.getOutputStream().write(new byte[1024]); // 发送一个1KB大小的数据包
            socket.getOutputStream().flush();
            long endSendTime = System.nanoTime();

            // 接收部分
            long startReceiveTime = System.nanoTime();
            socket.getInputStream().skip(1024); // 接收一个1KB大小的数据包
            long endReceiveTime = System.nanoTime();

            Duration sendDuration = Duration.ofNanos(endSendTime - startSendTime);
            long sendBandwidthBps = (long) (1024.0 / sendDuration.toMillis() * 8 * 1000); // 计算发送带宽（bps）
            System.out.println("Send bandwidth: " + sendBandwidthBps / 1024 + " KB/s"); // 输出发送带宽（KB/s）

            Duration receiveDuration = Duration.ofNanos(endReceiveTime - startReceiveTime);
            long receiveBandwidthBps = (long) (1024.0 / receiveDuration.toMillis() * 8 * 1000); // 计算接收带宽（bps）
            System.out.println("Receive bandwidth: " + receiveBandwidthBps / 1024 + " KB/s"); // 输出接收带宽（KB/s）
        } catch (IOException e) {
            e.printStackTrace();
        }

        //监控本地内存
        OperatingSystemMXBean osBean = ManagementFactory.getPlatformMXBean(OperatingSystemMXBean.class);
        long totalMemorySize = osBean.getTotalPhysicalMemorySize(); // 获取总物理内存大小
        long freeMemorySize = osBean.getFreePhysicalMemorySize(); // 获取空闲物理内存大小
        long usedMemorySize = totalMemorySize - freeMemorySize; // 计算已使用的物理内存大小

        System.out.println("Total memory: " + totalMemorySize / 1024 / 1024 + " MB"); // 输出总物理内存大小（MB）
        System.out.println("Used memory: " + usedMemorySize / 1024 / 1024 + " MB"); // 输出已使用的物理内存大小（MB）
    }
}
